
--
-- Fraese
--
-- @author: 	chrisu70
-- @date: 		27.09.2015
--

Fraese = {};

function Fraese.initSpecialization()
    
end;
function Fraese.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations) and SpecializationUtil.hasSpecialization(TurnOnVehicle, specializations) ;
end;

function Fraese:load(xmlFile)
	
    self.startFraese = SpecializationUtil.callSpecializationsFunction("startFraese");
    self.stopFraese = SpecializationUtil.callSpecializationsFunction("stopFraese");
    self.applyInitialAnimation = Utils.overwrittenFunction(self.applyInitialAnimation, Fraese.applyInitialAnimation);
    self.getDoRenderFillType = Utils.overwrittenFunction(self.getDoRenderFillType, Fraese.getDoRenderFillType);
    self.getIsTurnedOnAllowed = Utils.overwrittenFunction(self.getIsTurnedOnAllowed, Fraese.getIsTurnedOnAllowed);
    if self.isClient then
        self.FraeseStartSound = Utils.loadSample(xmlFile, {}, "vehicle.FraeseStartSound", nil, self.baseDirectory);
        self.FraeseWorkSound = Utils.loadSample(xmlFile, {}, "vehicle.FraeseWorkSound", nil, self.baseDirectory, self.components[1].node);
        self.FraeseStopSound = Utils.loadSample(xmlFile, {}, "vehicle.FraeseStopSound", nil, self.baseDirectory);
    end;
	self.fraeseStartAnimation = getXMLString(xmlFile, "vehicle.fraeseStartAnimation#name");
    self.fraeseStartAnimationSpeedScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fraeseStartAnimation#speedScale"), 1);
    self.fraeseStartAnimationInitialIsStarted = Utils.getNoNil(getXMLBool(xmlFile, "vehicle.fraeseStartAnimation#initialIsStarted"), false);
    self.fraeseTurnedOnRotationNodes = Utils.loadRotationNodes(xmlFile, {}, "vehicle.turnedOnRotationNodes.turnedOnRotationNode", "Fraese", self.components);
    self.getAllowFillShovel = Utils.overwrittenFunction(self.getAllowFillShovel, Fraese.getAllowFillShovel);
	self.isAlwaysTurnedOn = false;
	if self.shovelNodes ~= nil then
        self.isAlwaysTurnedOn = false;
    end
	self.lastLostFillLevel = 0;
	self.currentFillType = Fillable.FILLTYPE_UNKNOWN;
    self.lastValidFillType = Fillable.FILLTYPE_UNKNOWN;
    self.fraeseDirtyFlag = self:getNextDirtyFlag();
end;

function Fraese:applyInitialAnimation(superFunc)
    if self.fraeseStartAnimation ~= nil and self.playAnimation ~= nil and self.fraeseStartAnimationInitialIsStarted then
        self:playAnimation(self.fraeseStartAnimation, -self.fraeseStartAnimationSpeedScale, nil, true);
        AnimatedVehicle.updateAnimations(self, 99999999);
    end
    if superFunc ~= nil then
        superFunc(self);
    end
end
function Fraese:delete()
    if self.isClient then
        Utils.deleteSample(self.FraeseStartSound);
        Utils.deleteSample(self.FraeseStopSound);
        Utils.deleteSample(self.FraeseWorkSound);
    end;
	
end;
function Fraese:readStream(streamId, connection)
end;
function Fraese:writeStream(streamId, connection)
end;
function Fraese:readUpdateStream(streamId, timestamp, connection)
	
end;
function Fraese:writeUpdateStream(streamId, connection, dirtyMask)
	
end;
function Fraese:getSaveAttributesAndNodes(nodeIdent)
end;
function Fraese:mouseEvent(posX, posY, isDown, isUp, button)
end;
function Fraese:keyEvent(unicode, sym, modifier, isDown)
end;
function Fraese:update(dt)
    if self:getIsActive() then
        if self.isClient then
			if self.fillLevel >= self:getCapacity() and self:getCapacity() > 0 then
                self:setIsTurnedOn(false);
				self.attacherPipeVehicle = nil;
            end;
            if self:getIsTurnedOn() then
                if self:getIsActiveForSound() then
                    if not Utils.isSamplePlaying(self.FraeseStartSound, 1.8*dt) then
                        Utils.playSample(self.FraeseWorkSound, 0, 0, nil);
                    end;
                    Utils.stop3DSample(self.FraeseWorkSound);
                else
                    Utils.play3DSample(self.FraeseWorkSound);
                end
            end;
            
        end;
		if self.isClient then
        if self.shovelFillEffectTimeOut > 0 and self.shovelFillEffect ~= nil then
            self.shovelFillEffectTimeOut = self.shovelFillEffectTimeOut - dt;
            if self.shovelFillEffectTimeOut <= 0 then
                EffectManager:stopEffect(self.shovelFillEffect);
            end;
        end;
        if self.shovelEmptyEffectTimeOut > 0 and self.shovelEmptyEffect ~= nil then
            self.shovelEmptyEffectTimeOut = self.shovelEmptyEffectTimeOut - dt;
            if self.shovelEmptyEffectTimeOut <= 0 then
                EffectManager:stopEffect(self.shovelEmptyEffect);
            end;
        end;
    end;
        	
		if self.isClient then
			Utils.updateRotationNodes(self, self.fraeseTurnedOnRotationNodes, dt, self:getIsActive() and self:getIsTurnedOn());
		end;
	end;
end;
function Fraese:updateTick(dt)
    if self.isServer then
		if self:getCapacity() <= 0 then
            self.lastLostFillLevel = self.fillLevel;
            if self.fillLevel > 0 then
                self:setFillLevel(0, self.currentFillType);
            self:setIsTurnedOn(false);
			
			end
		end
						
	end;
end;
function Fraese:draw()
    if self.isClient then
        local percent = 0;
        if self:getCapacity() > 0 then
            percent = self.fillLevel/self:getCapacity()*100;
            if self.currentPipeState ~= nil and self.currentPipeState == 2 and not self.pipeIsUnloading and self.fillLevel > 0 then
                g_currentMission:addExtraPrintText(g_i18n:getText("Move_the_pipe_over_a_trailer"));
            elseif self.fillLevel == self:getCapacity() then
                g_currentMission:addExtraPrintText(g_i18n:getText("Dump_corn_to_continue_threshing"));
            end;
        end;
	end;
end;
function Fraese:onDeactivateSounds()
    if self.isClient then
        Utils.stopSample(self.FraeseWorkSound, true);
        Utils.stopSample(self.FraeseStartSound, true);
    end;
end;
function Fraese:startFraese()
    if self.isClient then
        if self.fraeseStartAnimation ~= nil and self.playAnimation ~= nil then
            self:playAnimation(self.fraeseStartAnimation, self.fraeseStartAnimationSpeedScale, self:getAnimationTime(self.fraeseStartAnimation), true);
        end
		self.displayEmptyInfo = true;
		self:getIsTurnedOn(true)
		self:setOverloadingActive(true)
		if self.isClient then
            Utils.stopSample(self.FraeseStopSound, true);
            if self:getIsActiveForSound() then
                Utils.playSample(self.FraeseStartSound, 1, 0, nil);
            end;
        end;
	end;
end;
function Fraese:stopFraese()
    if self.isClient then
        Utils.stopSample(self.FraeseWorkSound, true);
        Utils.stopSample(self.FraeseStartSound, true);
        Utils.stop3DSample(self.FraeseWorkSound);
        if self:getIsActiveForSound() then
            Utils.playSample(self.FraeseStopSound, 1, 0, nil);
        end
	end;
	if self.fraeseStartAnimation ~= nil and self.playAnimation ~= nil then
        self:playAnimation(self.fraeseStartAnimation, -self.fraeseStartAnimationSpeedScale, self:getAnimationTime(self.fraeseStartAnimation), true);
    end
end;
function Fraese:getIsTurnedOnAllowed(superFunc, isTurnedOn)
    if self.isAlwaysTurnedOn  then
        return false;
    end;
	if superFunc ~= nil then
        return superFunc(self, isTurnedOn);
    end;
    return true;
end;
function Fraese:onTurnedOn(noEventSend)
    self:startFraese();
end;
function Fraese:onTurnedOff(noEventSend)
    self:stopFraese();

end;
function Fraese:getAllowFillShovel(superFunc, fillType)
    if self:getIsTurnedOn() then
        if superFunc ~= nil then
            return superFunc(self, fillType);
        end
    end
    return false;
end;
function Fraese:getDoRenderFillType(superFunc)
    if not self.shovelShowFillTypeIcon or self.fillLevel == 0 then
        return false;
    end;
    if superFunc ~= nil then
        return superFunc(self);
    end
    return true;
end;
